#include "testpipeline.h"
#include <QtCore>

TestPipeline::TestPipeline(QObject *parent) :
    QObject(parent)
{
    timer = new QTimer(this);
    timer->setSingleShot(true);
    connect(this->timer,SIGNAL(timeout()),this,SLOT(timerSlot()));
    GError *err = NULL;
    pipe = gst_parse_launch("v4l2src ! videorate ! capsfilter name=vfps caps=\"video/x-raw-yuv,framerate=2/1\" ! fpsdisplaysink name=out-sink",&err);
    if (err !=NULL){
        qDebug()<<"Error initialing pipeline:"<<err->message;
        exit(1);
    }
    g_object_set(G_OBJECT(pipe),"name","pipe-element",NULL);
    GstBus *bus = gst_pipeline_get_bus(GST_PIPELINE(pipe));
    gst_bus_add_watch(bus, (GstBusFunc)on_message,this);
    gst_object_unref(bus);
    gst_element_set_state(pipe,GST_STATE_PLAYING);

}

void TestPipeline::changeFps()
{
    qDebug()<<"starting timer fps will be changed in 10 seconds";
    this->timer->start(10000);
}

void TestPipeline::timerSlot()
{
    qDebug()<<"change fps to 15 fps, look at the video overlay, you'll see 2 fps and not 15 ...";
    GstElement *vfps=gst_bin_get_by_name(GST_BIN(pipe),"vfps");
    GstCaps *new_caps; //=gst_caps_from_string("video/x-raw-yuv,framerate=15/1");
    new_caps = gst_caps_new_simple("video/x-raw-yuv",
                               "framerate",GST_TYPE_FRACTION, 15, 1,
                               NULL);
    g_object_set(G_OBJECT (vfps),"caps",new_caps,NULL);
    gst_caps_unref(new_caps);
    gst_object_unref(vfps);
}

gboolean TestPipeline::on_message(GstBus *bus, GstMessage *message, TestPipeline *data)
{
    Q_UNUSED(bus);
    switch (GST_MESSAGE_TYPE (message)) {
    case GST_MESSAGE_STATE_CHANGED:
    {
        QByteArray nome = "out-sink";
        if (GST_MESSAGE_SRC_NAME(message)==nome) {
            GstState old_state,new_state;
            gst_message_parse_state_changed(message,&old_state,&new_state,NULL);
            QString pipe_state = gst_element_state_get_name(new_state);
            qDebug()<<"new state for fpsdisplaysink:"<<pipe_state;
            if (pipe_state==QString("PLAYING")){
                data->changeFps();
            }
        }
        break;
    }
    case GST_MESSAGE_EOS:
    {
        qDebug()<<"EOS";
        QByteArray src_name = QByteArray(GST_MESSAGE_SRC_NAME(message));
        exit(1);
        break;
    }
    case GST_MESSAGE_ERROR:
    {
        qDebug()<<"ERROR";
        GError *err;
        gchar *debug;
        gst_message_parse_error(message, &err, &debug);
        qDebug()<<err->message<<debug;
        g_error_free(err);
        g_free(debug);
        exit(1);
        break;
    }
    default:
    {
        break;
    }
    }

    return TRUE;
}
